<?php
/**
 * dotcom-library functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * @package dotcom-library
 */

if ( ! defined( '_S_VERSION' ) ) {
	// Replace the version number of the theme on each release.
	define( '_S_VERSION', '1.0.0' );
}

/**
 * Sets up theme defaults and registers support for various WordPress features.
 *
 * Note that this function is hooked into the after_setup_theme hook, which
 * runs before the init hook. The init hook is too late for some features, such
 * as indicating support for post thumbnails.
 */
function dotcom_library_setup() {
	/*
		* Let WordPress manage the document title.
		* By adding theme support, we declare that this theme does not use a
		* hard-coded <title> tag in the document head, and expect WordPress to
		* provide it for us.
		*/
	add_theme_support( 'title-tag' );

	/*
		* Enable support for Post Thumbnails on posts and pages.
		*
		* @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
		*/
	add_theme_support( 'post-thumbnails' );
	add_theme_support('custom-logo');

	// This theme uses wp_nav_menu() in one location.
	register_nav_menus(
		array(
			'primary-menu' => esc_html__( 'Primary', 'dotcom-library' ),
			'footer-menu' => esc_html__( 'Footer', 'dotcom-library' ),
			'footer-services-menu' => esc_html__( 'Footer Services', 'dotcom-library' ),
			'policy-menu' => esc_html__( 'Policy Menu', 'dotcom-library' ),
		)
	);
}
add_action( 'after_setup_theme', 'dotcom_library_setup' );



/**
 * Enqueue scripts and styles.
 */
function dotcom_library_scripts() {
	wp_enqueue_style( 'dotcom-library-style', get_stylesheet_uri(), array(), _S_VERSION );
}
add_action( 'wp_enqueue_scripts', 'dotcom_library_scripts' );

/**Custom Code */

add_filter( 'acf/settings/rest_api_format', function () {
    return 'standard';
} );
function populate_acf_dropdown_with_theme_options($field) {
    // Clear existing choices to avoid duplicates
    $field['choices'] = array();
    // Get the field object for the 'theme_options' group
    $theme_options = get_field('theme_options', 'option');
    // Check if the field object and its value exist
    if ($theme_options && !empty($theme_options['theme_color_options'])) {
        $theme_options_values = $theme_options['theme_color_options'];

        // Loop through the theme_options values and populate the dropdown
        foreach ($theme_options_values as $key => $value) {
            // Replace underscores with spaces and capitalize each word for label
            $label = ucwords(str_replace('_', ' ', $key));
            // Add choice with label and value
            $field['choices'][$key] = $label ;
        }
    }
	switch ($field['name']) {
		case 'background_color':
			break;
		case 'text_color':
			break;
		default:
			$field['choices']['custom_color'] = 'Custom Color';
			break;
	}
		// 	// Add "Custom Color" option
		// 	$field['choices']['custom_color'] = 'Custom Color';

    return $field;
}
add_filter('acf/load_field/name=section_background_color', 'populate_acf_dropdown_with_theme_options');
add_filter('acf/load_field/name=color_dropdown', 'populate_acf_dropdown_with_theme_options');
add_filter('acf/load_field/name=background_color', 'populate_acf_dropdown_with_theme_options');
add_filter('acf/load_field/name=text_color', 'populate_acf_dropdown_with_theme_options');
// Hook into save_post to update the dropdown dynamically when theme_options are saved
function refresh_acf_dropdown_on_theme_options_save($post_id) {
    // Check if this is the options page and theme_options field group
    if ($post_id === 'options') {
        $theme_options = get_field('theme_options', 'option');
        if ($theme_options) {
            // Clear any cache or perform an action to refresh the field values
            delete_transient('acf_field_section_background_color_choices');
        }
    }
}
add_action('acf/save_post', 'refresh_acf_dropdown_on_theme_options_save', 20);

// Ensure the transient is cleared when the theme options are saved
add_action('acf/save_post', function($post_id) {
    if ($post_id === 'options') {
        delete_transient('acf_field_section_background_color_choices');
    }
}, 20);

// Step 1: Add the custom fields for Site Preview Link and Enable Redirect
function add_custom_general_fields() {
    // Register the new setting for Site Preview Link
    register_setting('general', 'site_preview_link', 'esc_url');

    // Add the new field for Site Preview Link
    add_settings_field(
        'site_preview_link',
        __('Site Preview Link'),
        'site_preview_link_field_html',
        'general',
        'default',
        array(
            'label_for' => 'site_preview_link'
        )
    );

    // Register the new setting for Enable Redirect
    register_setting('general', 'enable_redirect', 'sanitize_text_field');

    // Add the new field for Enable Redirect
    add_settings_field(
        'enable_redirect',
        __('Enable Redirect'),
        'enable_redirect_field_html',
        'general',
        'default'
    );
}

function site_preview_link_field_html() {
    $value = get_option('site_preview_link', '');
    echo '<input type="url" id="site_preview_link" name="site_preview_link" value="' . esc_attr($value) . '" class="regular-text ltr" />';
}

function enable_redirect_field_html() {
    $enable_redirect = get_option('enable_redirect', false);
    ?>
    <label for="enable_redirect">
        <input type="checkbox" id="enable_redirect" name="enable_redirect" <?php checked($enable_redirect, 'on'); ?> />
        Enable Redirect
    </label>
    <?php
}

add_action('admin_init', 'add_custom_general_fields');


// Step 2: Redirect URLs based on the custom field
function redirect_to_site_preview_link() {
	$enable_redirect = get_option('enable_redirect', false);

    if (!$enable_redirect) {
        return;
    }
	$site_preview_link = get_option('site_preview_link', '');

	if ($site_preview_link) {
		// Get the current URL
		$current_url = (is_ssl() ? 'https://' : 'http://') . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

		// Parse the current URL
		$parsed_url = wp_parse_url($current_url);

		// Get the path relative to the WordPress installation
		$site_url = get_site_url();
		$relative_path = str_replace($site_url, '', $current_url);

		// Build the new URL with the site preview link and the relative path
		$new_url = trailingslashit($site_preview_link) . ltrim($relative_path, '/');

		// Add query string if it exists
		if (!empty($parsed_url['query'])) {
			$new_url .= '?' . $parsed_url['query'];
		}

		// Redirect to the new URL
		wp_redirect($new_url, 301);
		exit();
	} 
}
add_action('template_redirect', 'redirect_to_site_preview_link');

/**Disabling image scaling */
add_filter( 'big_image_size_threshold', '__return_false' );


/**Pass TOTAL POSTS as custom header */
add_action('rest_api_init', function() {
    add_filter('rest_post_dispatch', 'add_custom_headers', 10, 3);
});

function add_custom_headers($result, $server, $request) {
    // Check if the request is for the posts endpoint
    if ($request->get_route() === '/wp/v2/posts') {
        // Get the query parameters from the request
        $query_params = $request->get_query_params();

        // Set up the query arguments for counting posts
        $query_args = array(
            'post_type' => 'post',
            'post_status' => 'publish',
            'fields' => 'ids',
            'posts_per_page' => -1, // Retrieve all posts
        );

        // Add category filtering if categories parameter is set
        if (isset($query_params['categories'])) {
            // Check if categories is a string or array
            if (is_string($query_params['categories'])) {
                $categories = explode(',', $query_params['categories']);
            } elseif (is_array($query_params['categories'])) {
                $categories = $query_params['categories'];
            }
            $query_args['category__in'] = $categories;
        }

        // Create a new query with the modified query arguments
        $query = new WP_Query($query_args);

        // Get the total number of posts for the modified query
        $total_posts = $query->found_posts;

        // Add custom headers
        $result->header('X-WP-Total', $total_posts);
    }

    return $result;
}

// Function to get all feeds from the database table
function get_all_feeds_from_db() {
    global $wpdb;
    // Query the wp_sbi_feeds table to get all feed IDs
    $results = $wpdb->get_results( "SELECT id,feed_name FROM {$wpdb->prefix}sbi_feeds" );
	// Initialize the output array
	$output = array();
	if (!empty($results)) {
		foreach ($results as $feed) {
			$output[$feed->id] = $feed->feed_name;
		}
	}
    return $output;
}

// Populate the ACF dropdown field with feeds from the database table
function populate_feed_selector_field( $field ) {
    // Get all feeds
    $feeds = get_all_feeds_from_db();

    // Reset choices
    $field['choices'] = array();

    // Add choices
    if ( !empty( $feeds ) ) {
        foreach ( $feeds  as $key => $value ) {
			$field['choices'][$key] = $value ;
        }
    }

    return $field;
}
add_filter( 'acf/load_field/name=social_feed', 'populate_feed_selector_field' );

add_action('rest_api_init', function() {
    add_filter('rest_page_query', 'filter_rest_page_query', 10, 2);
});

function filter_rest_page_query($args, $request) {
    if (isset($request['template'])) {
        $args['meta_query'] = array(
            array(
                'key' => '_wp_page_template',
                'value' => $request['template'],
                'compare' => '='
            )
        );
    }
    return $args;
}

// Adjust REST API query parameters to return all posts
function custom_rest_api_query($args, $request) {
    // Set a high per_page value if not already set
	if (!isset($_GET['per_page'])) {

		$args['posts_per_page'] = 100; // Return a high number of posts per page
	}
    return $args;
}

// Apply the filter for posts, pages, and custom post types
add_filter('rest_post_query', 'custom_rest_api_query', 10, 2);
add_filter('rest_page_query', 'custom_rest_api_query', 10, 2);

// Apply the filter for all custom post types
function apply_custom_rest_api_query_to_cpts() {
    $post_types = get_post_types(array('public' => true, '_builtin' => false), 'names');
    foreach ($post_types as $post_type) {
        add_filter("rest_{$post_type}_query", 'custom_rest_api_query', 10, 2);
    }
}
add_action('init', 'apply_custom_rest_api_query_to_cpts');

// Replace Wordpress url with preview link in yoast seo
function replace_home_url_with_preview($content) {
    $site_preview_link = trailingslashit(get_option('site_preview_link', home_url()));
    $home_url = trailingslashit(home_url());
    
    if (is_array($content)) {
        // Traverse the schema data array and replace URLs
        array_walk_recursive($content, function (&$value) use ($home_url, $site_preview_link) {
            if (is_string($value) && filter_var($value, FILTER_VALIDATE_URL)) {
                $value = str_replace($home_url, $site_preview_link, $value);
            }
        });
        return $content;
    } elseif (is_string($content)) {
        return str_replace($home_url, $site_preview_link, $content);
    }

    return $content;
}
// Update Meta Title
add_filter('wpseo_title', 'replace_home_url_with_preview');

// Update Meta Description
add_filter('wpseo_metadesc', 'replace_home_url_with_preview');

// Update Canonical URL
add_filter('wpseo_canonical', 'replace_home_url_with_preview');

// Update Open Graph URL
add_filter('wpseo_opengraph_url', 'replace_home_url_with_preview');

// Update Twitter Card URL
add_filter('wpseo_twitter_url', 'replace_home_url_with_preview');

// Update Sitemap URL
add_filter('wpseo_sitemap_url', 'replace_home_url_with_preview', 10, 2);

// Update Schema URL
add_filter('wpseo_schema_graph', 'replace_home_url_with_preview', 10, 1);



/**Add Version setting in all ACF Fields */
// Hook into ACF to add the version setting to all field types
add_action('acf/render_field_settings', 'add_version_setting_to_all_fields', 10, 1);

function add_version_setting_to_all_fields($field) {
    // Add a version setting field
    acf_render_field_wrap(array(
        'label' => __('Version', 'acf'),
        'instructions' => __('Enter the version for this field.', 'acf'),
        'type' => 'text',
        'name' => 'version',
        'prefix' => 'acf_field[' . $field['ID'] . ']',
        'value' => isset($field['version']) ? $field['version'] : '', // Default to the current version or empty
    ));
}

// Hook into ACF to save the version setting
add_filter('acf/update_field', 'save_version_setting', 10, 1);

function save_version_setting($field) {
    // Update the field with the version setting
    if (isset($_POST['acf_field'][$field['ID']]['version'])) {
        $field['version'] = sanitize_text_field($_POST['acf_field'][$field['ID']]['version']);
    }
    return $field;
}

// Hook into ACF to load the version setting when editing the field
add_filter('acf/load_field', 'load_version_setting', 10, 1);

function load_version_setting($field) {
    // Ensure the version setting is loaded
    if (!isset($field['version'])) {
        $field['version'] = ''; // Default to empty if not set
    }
    return $field;
}

// Hook into ACF to render the field with the version value
add_filter('acf/prepare_field', 'render_field_with_version', 10, 1);

function render_field_with_version($field) {
    // Add version data attribute to the field wrapper
    if (isset($field['version']) && !empty($field['version'])) {
        $field['wrapper']['data-version'] = $field['version'];
    }
    return $field;
}




//add SVG to allowed file uploads
function add_file_types_to_uploads($file_types){

     $new_filetypes = array();
     $new_filetypes['svg'] = 'image/svg';
     $file_types = array_merge($file_types, $new_filetypes );

     return $file_types; 
} 
add_action('upload_mimes', 'add_file_types_to_uploads');